// [License]
// The Ariba-Underlay Copyright
//
// Copyright (c) 2008-2009, Institute of Telematics, Universität Karlsruhe (TH)
//
// Institute of Telematics
// Universität Karlsruhe (TH)
// Zirkel 2, 76128 Karlsruhe
// Germany
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE INSTITUTE OF TELEMATICS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INSTITUTE OF TELEMATICS OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation
// are those of the authors and should not be interpreted as representing
// official policies, either expressed or implied, of the Institute of
// Telematics.
// [License]

#ifndef COMMUNICATIONLISTENER_H_
#define COMMUNICATIONLISTENER_H_

#include "Message.h"
#include "Identifiers.h"
#include "LinkProperties.h"
#include "DataMessage.h"

namespace ariba {

// forward decl
namespace overlay {
	class BaseOverlay;
}

/**
 * Listener for communication events on links.
 */
class CommunicationListener {
	friend class ariba::overlay::BaseOverlay;
protected:
	CommunicationListener();
	virtual ~CommunicationListener();

	// --- link events ---

	virtual void onLinkUp(const LinkID& lnk, const NodeID& remote);

	virtual void onLinkDown(const LinkID& lnk, const NodeID& remote);

	virtual void onLinkChanged(const LinkID& lnk, const NodeID& remote);

	virtual void onLinkFail(const LinkID& lnk, const NodeID& remote);

	virtual bool onLinkRequest(const NodeID& remote, const DataMessage& msg = DataMessage::UNSPECIFIED);

	// --- general receive method ---

	virtual void onMessage(const DataMessage& msg, const NodeID& remote,
			const LinkID& lnk = LinkID::UNSPECIFIED);

	// --- extended message functionality ---
	// virtual void onLinkQoSChanged(const LinkID& lnk, const NodeID& remote,
	//			const LinkProperties& prop);

	// --- extended message functionality ---
	//	virtual void onMessageSent(seqnum_t seq_num, bool failed,
	//		const DataMessage& msg = DataMessage::UNSPECIFIED);

	// --- dht functionality ---
	//	virtual void onGetResponse( const Identifier<> id, const Message* msg );
	//	virtual void onPutResponse( const Identifier<> id, const Message* msg );
};

} // namespace ariba

#endif /* COMMUNICATIONLISTENER_H_ */
